package Depot::Conf;

# Conf.pm -- author: Oliver Bossert
# Copyright (C) 2003 Oliver Bossert

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use vars qw(%conf);

############################################################
sub load {
############################################################
  my ($file) = @_;
  $file = $ENV{'HOME'} . "/.depot/config" if ( !defined($file) );

  return if (! -e $file);

  open (FILE, "< $file") or die "Can not open $file: $!\n";
  while (<FILE>) {
    chomp;
    next if /^\s*#/;
    next if /^\s*$/;
    s/^\s*//; s/\s*$//;
    my ($key, $val) = split /\s+/, $_, 2;
    $conf{lc($key)} = $val;
  }
  close FILE;
}

############################################################
sub clear { 
############################################################
  %conf = () 
}

############################################################
sub save {
############################################################
  my ($file) = @_;
  $file = $ENV{'HOME'} . "/.depot/config" if ( !defined($file) );

  open (FILE, "> $file") || die "Can not write to $file: $!\n";
  foreach (sort keys %conf) {
    print FILE $_ . "\t" . $conf{$_} . "\n";
  }
  close FILE;
}

############################################################
sub get {
############################################################
  return $conf{lc($_[0])}; 
}

############################################################
sub set { 
############################################################
  $conf{lc($_[0])} = $_[1] 
}

############################################################
sub default {
############################################################
  my ($key, $val) = @_;
  $key = lc($key);
  if (! defined($conf{$key})) {
    $conf{$key} = $val;
  }
}

############################################################
sub get_first {
############################################################
  my (@keys) = @_;
  foreach (@keys) {
    my $value = get($_);
    if (defined($value) && $value) {
      return $value;
    }
  }
  return "";
}

1;
