package Depot::Layout;

# Layout.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use CGI qw/:standard/;
use CGI::Carp qw(fatalsToBrowser);

use Depot::Conf;
use Depot::Tools;

############################################################
sub new { # Generate a new Object
############################################################
  my $type = shift;
  my $data = shift;
  my $class = ref($type) || $type;
  my $self = {};
  bless $self, $class;

  Depot::Conf::default("Layout::BG1", "#C0C0C0");
  Depot::Conf::default("Layout::Title", "Olf's personal Depot");

  $self->{'data'} = $data;

  return $self;
}

############################################################
sub print_site { # 
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  $erg .= $query->header;
  $erg .=  $query->start_html( -title => Depot::Conf::get("Layout::Title"),
			       -BGCOLOR => Depot::Conf::get("Layout::BG1") );

  my $depotname = "";
  $depotname = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'}->{'name'} if (defined($query->param('depot')) &&
											    $query->param('depot') ne "");
  $erg .= $query->table(
			$query->TR(
				   $query->td( { -width => '75%' }, 
					       $query->h1( Depot::Conf::get("Layout::Title") . ": " . 
							   $depotname )),
				   $query->td( { -width => '25%' }, 
					       $self->depot_selection( $query ) )
				  )
		       );;

  # Add the actual depot-value
  $erg .= $self->actual_cash( $query );

  # Add the open positions
  $erg .= $self->open_positions( $query );

  # Add the open orders
  $erg .= $self->open_orders( $query );

  # Add new order
  $erg .= $self->new_order( $query );

  # Add the history
  $erg .= $self->history( $query );

  $erg .= end_html;
  return $erg;
}


############################################################
sub depot_selection { # 
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";

  my %labels = ();
  my @values = ();
  foreach my $depot ( keys %{$self->{'data'}->{'depots'}} ) {
    push @values, $depot;
    $labels{$depot} = $self->{'data'}->{'depots'}{$depot}->{'depot'}->{'name'};
  }

  # Select an existing depot
  my $default = $query->param('depot') || $values[0];
  $erg.= $query->start_form();
  $erg.= $query->popup_menu( -name => "depot",
			     -default => $default,
			     -values => \@values,
			     -labels => \%labels );
  $erg .= $query->submit("Change");
  $erg .= $query->end_form();

  # Create a new depot
  $erg .= $query->start_form();
  $erg .= $query->textfield( -name => "newdepot",
			     -default => "New Depot"
			   );
  $erg .= $query->submit("New");
  $erg .= $query->end_form();

  return $erg;
}


############################################################
sub actual_cash { # Print the actual value of the depot
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  my $depot;
  $depot = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'} if (defined($query->param('depot')));

  return $erg if ( !defined($depot) || $depot eq "" );

  my $tablecontent = "";
  $tablecontent .= TR( td("Aktueller Cash: "), 
		       td({-align => 'right', -width => '250'}, 
			  sprintf("%.2f EUR", $depot->{'cash'}->{'value'})));

  my $depotsum = 0;
  foreach my $p (  @{$depot->{'positions'}->{'position'}} ) {
    if ( $p->{'type'} =~/long/i ) {
      $depotsum += getval( $p->{'wkn'} ) * $p->{'nb'};
    } else {
      $depotsum += $p->{'nb'} * ($p->{'open'}) + ( $p->{'nb'} *( $p->{'open'} - getval($p->{'wkn'}) ) );
    }
  }
  $tablecontent .= TR( td("Aktuelle Depotsumme: "), 
		       td({-align => 'right'}, 
			  sprintf("%.2f EUR", $depotsum)) );
  $tablecontent .= TR( {-bgcolor => 'white' }, 
		       td(b("Gesamter Wert: ")), 
		       td({-align => 'right'}, 
			  b(sprintf("%.2f EUR", ($depotsum+$depot->{'cash'}->{'value'})))) );

  $erg = table( {-border=>0,-cellpadding=>5,-cellspacing=>0}, 
		$tablecontent);

  return $erg;
}

############################################################
sub open_positions { # Print the open positions
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  my $depot;
  $depot = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'} if (defined($query->param('depot')));

  return $erg if ( !defined($depot) || $depot eq "" );

  $erg .= h2("Open Positions");
  my $tablecontent = "";
  $tablecontent = TR( {-bgcolor=>'#8080FF'},
		      th("Datum"),
		      th("WKN"),
		      th("L/S"),
		      th("Anzahl"),
		      th("Kaufkurs"),
		      th("Aktueller Kurs"),
		      th("Prozent"),
		      th("Close") );
  my $color = 0;


  foreach my $p ( sort {$a->{'date'} cmp $b->{'date'}} @{$depot->{'positions'}->{'position'}} ) {

    my $rowcontent = "";

    my $temp = $p->{'date'};
    $temp =~ s/_/&nbsp;/;
    $rowcontent .= td($temp);

    $rowcontent .= td($p->{'wkn'});
    $rowcontent .= td($p->{'type'});
    $rowcontent .= td($p->{'nb'});
    $rowcontent .= td(sprintf("%.2f",$p->{'open'}));

    # Aktueller Kurs
    $temp = getval( $p->{'wkn'} );
    $rowcontent .= td(sprintf("%.2f",$temp));

    # Prozent
    my $prozent = ( 100 * ( ($p->{'nb'}*$temp) - ($p->{'nb'}*$p->{'open'}) ) ) / ($p->{'nb'}*$p->{'open'});
    $prozent *= -1 if ($p->{'type'} =~ /short/i);
    my $col = "#000000";
    $col = "#AA0000" if ($prozent < 0);
    $col = "#00AA00" if ($prozent > 0);
    $rowcontent .= td(span({-style=>"Color: $col;"}, sprintf("%.2f",$prozent)));

    # Close
    $rowcontent .= td( start_form(),
		       hidden('depot', $query->param('depot')),
		       hidden('close',1),
		       hidden('id',$p->{'id'}),
		       submit("CLOSE"),
		       end_form() );

    $tablecontent .= TR({-bgcolor=>'#DDDDDD'}, 
			$rowcontent) if ($color);
    $tablecontent .= TR($rowcontent) if (!$color);

    $color = !$color;
  }
  $erg .= table( {-border=>0,-cellpadding=>5,-cellspacing=>0}, 
		 $tablecontent );

  return $erg;
}

############################################################
sub open_orders { # Print the open orders
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  my $depot;
  $depot = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'} if (defined($query->param('depot')));

  return $erg if ( !defined($depot) || $depot eq "" );

  $erg.= h2("Open Orders");
  my $tablecontent = TR( {-bgcolor=>'#8080FF'}, 
			 th("Datum"), 
			 th("WKN"), 
			 th("L/S"), 
			 th("Anzahl / Preis"), 
			 th("Bedingung"), 
			 th("Aktueller Kurs"), 
			 th("Cancel?") );
  my $color = 0;
  foreach my $p ( sort {$a->{'date'} cmp $b->{'date'}} @{$depot->{'orderlist'}->{'order'}} ) {

    my $rowcontent = "";
    my $temp = $p->{'date'};
    $temp =~ s/_/&nbsp;/;
    $rowcontent .= td($temp);

    $rowcontent .= td($p->{'wkn'});
    $rowcontent .= td($p->{'type'});
    $rowcontent .= td($p->{'nb'});
    # Bedingung
    $temp = $p->{'if'}->{'name'} . " (";
    my @tmp = ();
    foreach my $param ( @{$p->{'if'}->{'param'}} ) {
      push @tmp,  $param->{'name'} . " => " . $param->{'value'};
    }
    $temp .= join(",", @tmp) . ")";
    $rowcontent .= td($temp);

    # Aktueller Kurs
    $temp = getval( $p->{'wkn'} );
    $rowcontent .= td(sprintf("%.2f",$temp));

    # Button fr Schlieen
    $rowcontent .= td( start_form(),
		       hidden('depot', $query->param('depot')),
		       hidden('cancel',1),
		       hidden('id',$p->{'id'}),
		       submit("CANCEL"),
		       end_form() );
    $tablecontent .= TR({-bgcolor=>'#DDDDDD'}, 
			$rowcontent) if ($color);
    $tablecontent .= TR($rowcontent) if (!$color);
    $color = !$color;
  }
  $erg .= table( {-border=>0,-cellpadding=>5,-cellspacing=>0}, 
		 $tablecontent );
  return $erg;
}

############################################################
sub new_order { # Print the open orders
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  my $depot;
  $depot = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'} if (defined($query->param('depot')));

  return $erg if ( !defined($depot) || $depot eq "" );

  $erg .= h2("New Order");
  $erg .= start_form;
  $erg .= hidden('depot', $query->param('depot'));
  $erg .= table(
		TR(td("WKN:"), td(textfield('wkn'))),
		TR(td("Buy/Sell:"), td(radio_group(-name=>'buysell',
						   -values=>['buy','sell'],
						   -default=>'buy',
						   -linebreak=>'true'))),
		TR(td("Anzahl/Preis:"), td(textfield('count'))),
		TR(td("Bedingung:"), td(textfield('func'))),
		  hidden('neworder',1), 
		TR(td(), td(submit("Order aufgeben"))));
  $erg .= end_form;

  return $erg;
}

############################################################
sub history { # Print the open orders
############################################################
  my $self = shift;
  my $query = shift;
  my $erg = "";
  my $depot;
  $depot = $self->{'data'}->{'depots'}{$query->param('depot')}->{'depot'} if (defined($query->param('depot')));

  return $erg if ( !defined($depot) || $depot eq "" );

  $erg .= h2("History");
  my $tablecontent = TR( {-bgcolor=>'#8080FF'},
			 th("WKN"),
			 th("L/S"),
			 th("Datum"),
			 th("Anzahl"),
			 th("Kurs"),
			 th("Wert"),
			 th("Prozent") );

  my $color = 0;
  if ( defined(@{$depot->{'history'}->{'trade'}}) ) {
    foreach my $p ( sort {$b->{'close'}->{'date'} cmp $a->{'close'}->{'date'}} @{$depot->{'history'}->{'trade'}} ) {

      my $rowcontent = "";
      $rowcontent .= td($p->{'wkn'});
      $rowcontent .= td($p->{'type'});

      my $temp = $p->{'open'}->{'date'};
      $temp =~ s/_/&nbsp;/;
      $rowcontent .= td($temp);
      $rowcontent .= td($p->{'nb'});
      $rowcontent .= td(sprintf("%.2f",$p->{'open'}->{'value'}));
      $rowcontent .= td(sprintf("%.2f",($p->{'nb'}*$p->{'open'}->{'value'})));
      $rowcontent .= td();

      $tablecontent .= TR({-bgcolor=>'#DDDDDD'}, $rowcontent) if ($color);
      $tablecontent .= TR($rowcontent) if (!$color);

      $rowcontent = "";
      $rowcontent .= td();
      $rowcontent .= td();
      $temp = $p->{'close'}->{'date'};
      $temp =~ s/_/&nbsp;/;
      $rowcontent .= td($temp);
      $rowcontent .= td($p->{'nb'});
      $rowcontent .= td(sprintf("%.2f",$p->{'close'}->{'value'}));
      $rowcontent .= td(sprintf("%.2f",($p->{'nb'}*$p->{'close'}->{'value'})));
      # Prozent
      my $prozent = ( 100 * ( ( $p->{'nb'} * $p->{'close'}->{'value'} ) - 
			      ( $p->{'nb'} * $p->{'open'}->{'value'} ) ) ) / 
				($p->{'nb'} * $p->{'open'}->{'value'});
      $prozent *= -1 if ($p->{'type'} =~ /short/i);

      my $col = "#000000";
      $col = "#AA0000" if ($prozent < 0);
      $col = "#00AA00" if ($prozent > 0);
      $rowcontent .= td(span({-style=>"Color: $col;"}, 
			     sprintf("%.2f",$prozent)));

      $tablecontent .= TR({-bgcolor=>'#DDDDDD'}, 
			  $rowcontent) if ($color);
      $tablecontent .= TR($rowcontent) if (!$color);


      if ( defined( $p->{'fee'}->{'value'} ) ) {
	$prozent = ( 100 * ( ( $p->{'nb'} * $p->{'close'}->{'value'} ) - 
			     ( $p->{'nb'} * $p->{'open'}->{'value'} ) -
			     $p->{'fee'}->{'value'} ) ) / 
			       ($p->{'nb'}*$p->{'open'}->{'value'});
	if ($p->{'type'} =~ /short/i) {
	  $prozent = ( -100 * ( ( $p->{'nb'} * $p->{'close'}->{'value'} ) - 
			       ( $p->{'nb'} * $p->{'open'}->{'value'} ) +
			       $p->{'fee'}->{'value'} ) ) / 
				 ($p->{'nb'}*$p->{'open'}->{'value'});
	}
	$col = "#AA0000" if ($prozent < 0);
	$col = "#00AA00" if ($prozent > 0);
	$rowcontent = td().td().td("Fee").td().td().td(sprintf("%.2f",$p->{'fee'}->{'value'}));
	$rowcontent .= td(span({-style=>"Color: $col;"}, 
			       sprintf("%.2f",$prozent)));

	$tablecontent .= TR({-bgcolor=>'#DDDDDD'}, 
			    $rowcontent) if ($color);
	$tablecontent .= TR($rowcontent) if (!$color);
      }

      foreach my $c ( @{$p->{'comment'}} ) {
	my $tc = span({-style=>"font-size:0.9em;"}, 
		      $c->{'value'});
	$tablecontent .= TR( {-bgcolor=>'#DDDDDD'}, 
			     td() . td() . 
			     td( {-colspan => 5}, $tc) ) if ($color);
	$tablecontent .= TR( td() . td() . 
			     td( {-colspan => 5}, $tc) ) if (!$color);
      }


      my $tc = start_form .
	hidden('depot', $query->param('depot')).
	  textfield('comment') . 
	    hidden('id',$p->{'id'}) .
	      submit('Kommentar') .
		end_form;

      $tablecontent .= TR( {-bgcolor=>'#DDDDDD'}, td() . td() . 
			   td( {-colspan => 5}, $tc) ) if ($color);
      $tablecontent .= TR( td() . td() . 
			   td( {-colspan => 5}, $tc) ) if (!$color);

      $color = !$color;

    }
  }
  $erg .= table( {-border=>0,-cellpadding=>5,-cellspacing=>0}, $tablecontent );

  return $erg;
}



1;

